@@ -1,24 +1,18 @@
 #ifndef PERLIO_UTIL_H
 #define PERLIO_UTIL_H
 
-#define  PERLIO_FUNCS_CONST
-
 #include "EXTERN.h"
 #include "perl.h"
 #include "XSUB.h"
 
 #include "perliol.h"
 
+#include "ppport.h"
+
 #define LayerFetch(layer, n) ((layer)->array[n].funcs)
 #define LayerFetchSafe(layer, n) ( ((n) >= 0 && (n) < (layer)->cur) \
 				? (layer)->array[n].funcs : PERLIO_FUNCS_CAST(&PerlIO_unix) )
 
-#ifndef PERLIO_FUNCS_DECL
-#define PERLIO_FUNCS_DECL(funcs) const PerlIO_funcs funcs
-#define PERLIO_FUNCS_CAST(funcs) (PerlIO_funcs*)(funcs)
-#endif
-
-#include "ppport.h"
 
 #define IOLflag(f, flag)     (PerlIOBase((f))->flags & (flag))
 #define IOLflag_on(f, flag)  (PerlIOBase((f))->flags |= (flag))
